#-*- coding: utf-8 -*-

"""
    main.py
    ~~~~~~~

    Waltz is a web.py framework for designing web apps in 3/4
    time. Waltz comes pre-configured and includes features like
    out-of-the-box support for analytics tracking. Waltz and never miss
    a beat.

    :copyright: (c) 2012 by Mek
    :license: BSD, see LICENSE for more details.
"""

import web
import pickle
from lazydb.lazydb import Db
from copy import copy
from reloader import PeriodicReloader
from configs.config import server

urls = ('/analytics', 'Analytics',
        '/', 'Index',
        '/404', 'NotFound',
        '(.*)', 'NotFound')

app = web.application(urls, globals(), autoreload=False)

_globals = {'ctx': web.ctx}
slender  = web.template.render('templates/', globals=_globals)
render  = web.template.render('templates/', base='base', globals=_globals)

def track(fn):
    """A decorator which wraps each route with analytics tracking."""
    def tracked(fn):
        """This setup w/ second inner function allows support for
        passing @track optional arguments and parameters, as well as
        allows for the special case of class methods which require
        self - methods(self).
        """        
        def inner(*args, **kwargs):
            """Copy web context environment, clean it to avoid
            pickeling issues, and dump it to lazydb before returning
            the route
            """
            db = Db('database')            
            ctx = copy(web.ctx['env'])
            del ctx['wsgi.errors']
            del ctx['wsgi.input']
            db.append('analytics', ctx)
            return fn(*args, **kwargs)        
        return inner
    return tracked(fn)

class Index:

    @track
    def GET(self):
        return render.index()

class Analytics:

    def GET(self):
        """Add your own key in server config"""
        i = web.input(key="")
        if i.key == server['secret']:
            db = Db('database')
            return db.get('analytics')

class NotFound:
    def GET(self, err=None):
        raise web.notfound('')

if __name__ == "__main__":
    if server['debug_mode']:
        PeriodicReloader()
    app.run()
